rm(list=ls(all=TRUE))
library(MASS)

#generate survival time
# par is the number of partition to stepwise function of X, the random covariate is stored in a column vector rcovariate, 
# u0 is a uniform random variable, # k0 indicates which subject, node means accumulated partition length.
inversef=function(u0,k0){
  step=rep(0,par)
  co=rcovariate[((k0-1)*par+1):(k0*par)]
  step[1]=a*exp(beta.real*co[1])*node[1]
  for(i in 2:par)
     step[i]=step[i-1]+a*exp(beta.real*co[i])*(node[i]-node[i-1])
  temp=-log(u0)
  count=1
  while ((temp>step[count])&&(count<par)){
    count=count+1}
  if (count==1)
      return(temp/(a*exp(beta.real*co[1])))
  if(count>1)
      return((temp-step[count-1])/(a*exp(beta.real*co[count]))+node[count-1])
}


id=function(a,b){
 y=b
 y[b<=a]=1
 y[b>a]=0
 return(y)
}

yt=function(x0,u0){
  y=x0
  y[u0<=x0]=1
  y[u0>x0]=0
  return(y)
}

Kh=function(h,t0){
  y=t0
  y[abs(t0/h)>=1]=0
  y[abs(t0/h)<1]=0.75*(1-(t0[abs(t0/h)<1]/h)^2)/h
  return(y)
}

# u indicates uncensored death time, observe means the observed measurement time, xx = min(T, C)
s0=function(u,beta){
  return(sum(Kh(h,u-observe)*id(u,observe)*yt(xx,u)*exp(beta*covariate)))
}
  

s1=function(u,beta){
  return(sum(Kh(h,u-observe)*id(u,observe)*yt(xx,u)*covariate*exp(covariate*beta)))
}

s2=function(u,beta){
  return(sum(Kh(h,u-observe)*id(u,observe)*yt(xx,u)*(covariate^2)*exp(beta*covariate)))
}

ss0=function(u,beta){
  return(sum(Kh(h,u-observe)*yt(xx,u)*exp(beta*covariate)))
}
  

ss1=function(u,beta){
  return(sum(Kh(h,u-observe)*yt(xx,u)*covariate*exp(covariate*beta)))
}

ss2=function(u,beta){
  return(sum(Kh(h,u-observe)*yt(xx,u)*(covariate^2)*exp(beta*covariate)))
}


#num is the number of repeated measurments for subject i, delta is the failure indicator and xi is the death time
#for the ith subject, ord records the location of the ith subject in the covariate vector.
#New-Ralph method
f=function(beta){
 L1=0
 L2=0
 for (i in 1:n){
   if((num[i]>0)&&(delta[i]==1)&&(x[i]<=tau)){
   from=ord[i]+1
   to=ord[i]+num[i]
   include=from:to
   temp0=s0(x[i],beta)
   temp1=s1(x[i],beta)
   temp2=s2(x[i],beta)
if (temp0==0)next
   L1=L1+sum(Kh(h,x[i]-observe[include])*id(x[i],observe[include])*(covariate[include]-temp1/temp0))
   L2=L2+sum(Kh(h,x[i]-observe[include])*id(x[i],observe[include])*(temp1^2-temp2*temp0)/(temp0^2))
   }
 }
return(beta-L1/L2)
}
   
#covariance estimation
fse=function(beta){
 L11=0
 L2=0
 for (i in 1:n){
   if((num[i]>0)&&(delta[i]==1)&&(x[i]<=tau)){
   from=ord[i]+1
   to=ord[i]+num[i]
   include=from:to
   temp0=s0(x[i],beta)
   temp1=s1(x[i],beta)
   temp2=s2(x[i],beta)
if (temp0==0) next
   L11=L11+(sum(Kh(h,x[i]-observe[include])*id(x[i],observe[include])*(covariate[include]-temp1/temp0)))^2
   L2=L2+sum(Kh(h,x[i]-observe[include])*id(x[i],observe[include])*(temp1^2-temp2*temp0)/(temp0^2))
   }
 }
return(sqrt(L11/(L2^2)))
}

##################################main program starts########################
beta=NULL
se=NULL
indicator=NULL
censoring=NULL

beta.last = NULL
se.last = NULL
indicator.last = NULL


const=5.1
a=2
tau=1
beta.real=1.5
par=20
node=seq(1/par,1,1/par)
lambda = 8

n=100
h=seq(from = n^(-0.7), n^(-0.6), length = 30)
lbd = length(h)
x=rep(0,n)
num.rep = 1
while(num.rep <= 1000){
  delta=rep(1,n)
 # print(k)
  flag=0
  while (flag==0){
    u=runif(n)
    num=rpois(n,lambda)
    observe=runif(min = 0, max = 1, sum(num))
    ord=1:n
    ord[1]=0
    
    for(i in 2:n){
      ord[i]=sum(num[1:(i-1)])
          }
    covariate=rep(0,sum(num))
    mu=rep(0,par)
    sigma=rep(1,par)
    sigma=diag(sigma)
    for(j in 1:(par-1)){
       for(k in (j+1):par){
         sigma[j,k]=exp(-abs(j-k)/par)
         sigma[k,j]=sigma[j,k]
       }
    }
    rcovariate.1=mvrnorm(n,mu,sigma)
    rcovariate=t(rcovariate.1)
    rcovariate=as.vector(rcovariate)
    t=rep(0,n)
    for(i in 1:n){
  #    t[i]=inversef(u[i],i)
  step=rep(0,par)
  co=rcovariate[((i-1)*par+1):(i*par)]
  step[1]=a*exp(beta.real*co[1])*node[1]
  for(j in 2:par)
     step[j]=step[j-1]+a*exp(beta.real*co[j])*(node[j]-node[j-1])
  temp=-log(u[i])
  count=1
  while ((temp>step[count])&&(count<par)){
    count=count+1}
  if (count==1)
      t[i] = temp/(a*exp(beta.real*co[1]))
  if(count>1)
      t[i] = (temp-step[count-1])/(a*exp(beta.real*co[count]))+node[count-1]
	}
    if (min(t)>min(observe)&&sum(num>0)==length(num))
      flag=1
 }
# we finish the generation of event time and random covariate.
# extract observed covariate for different subjects.
 for(i in 1:n){
   temp1=rcovariate[((i-1)*par+1):(i*par)]
   temp2=observe[(ord[i]+1):(ord[i]+num[i])]
   covariate[(ord[i]+1):(ord[i]+num[i])]=temp1[floor(temp2*par)+1]
 }
  censor=const*runif(n)
   x=t
  x[censor<t]=censor[censor<t]
  delta[censor<t]=0
  censoring=c(censoring,1-sum(delta)/n)
  xx=1:sum(num)
  if(ord[2]>0){
    xx[1:ord[2]]=x[1]}
    for(i in 2:n){
    if (num[i]!=0){
      xx[(ord[i]+1):(ord[i]+num[i])]=x[i]
    }
  }

################################################################
### now we start the inference part########################

beta_hat.0 = rep(0, lbd)
beta_hat.1 = rep(0, lbd)
beta_hat.2 = rep(0, lbd)


hatv.1 = rep(0, lbd)


cvlabel = sample(rep(c(1:2), length =n ))

for (bd in 1:lbd){

   beta0=0.01
  sign=0
  while(sign==0){
 L1=0
 L2=0

 for (i in 1:n){
   if((num[i]>0)&&(delta[i]==1)&&(x[i]<=tau)){
   from=ord[i]+1
   to=ord[i]+num[i]
   include=from:to

temp00 = sum(Kh(h[bd],x[i]-observe)*id(x[i],observe)*yt(xx,x[i]) * exp(beta0*covariate))
temp11 = sum(Kh(h[bd],x[i]-observe)*id(x[i],observe)*yt(xx,x[i]) * covariate * exp(beta0*covariate))
temp22 = sum(Kh(h[bd],x[i]-observe)*id(x[i],observe)*yt(xx,x[i]) * covariate^2 * exp(beta0*covariate))

if(temp00 == 0) next
   L1=L1+sum(Kh(h[bd],x[i]-observe[include])*id(x[i],observe[include])*(covariate[include]-temp11/temp00))
   L2=L2+sum(Kh(h[bd],x[i]-observe[include])*id(x[i],observe[include])*(temp11^2-temp22*temp00)/(temp00^2))
   }
 }

if (is.na(L2)) return
beta.hat = beta0-L1/L2


 #   beta.hat=f(beta0)
    if (abs(beta0-beta.hat)<10^-3)
      sign=1
    beta0=beta.hat
  }
beta_hat.0[bd] = beta0

 beta0=0.01
  sign=0
  while(sign==0){
 L1=0
 L2=0

 for (i in 1:n){ if (cvlabel[i]==2) next
   if((num[i]>0)&&(delta[i]==1)&&(x[i]<=tau)){
   from=ord[i]+1
   to=ord[i]+num[i]
   include=from:to

temp00 = sum(Kh(h[bd],x[i]-observe)*id(x[i],observe)*yt(xx,x[i]) * exp(beta0*covariate))
temp11 = sum(Kh(h[bd],x[i]-observe)*id(x[i],observe)*yt(xx,x[i]) * covariate * exp(beta0*covariate))
temp22 = sum(Kh(h[bd],x[i]-observe)*id(x[i],observe)*yt(xx,x[i]) * covariate^2 * exp(beta0*covariate))

if(temp00 == 0) next
   L1=L1+sum(Kh(h[bd],x[i]-observe[include])*id(x[i],observe[include])*(covariate[include]-temp11/temp00))
   L2=L2+sum(Kh(h[bd],x[i]-observe[include])*id(x[i],observe[include])[]*(temp11^2-temp22*temp00)/(temp00^2))
   }
 }

if (is.na(L2)) return
beta.hat = beta0-L1/L2
    if (abs(beta0-beta.hat)<10^-3)
      sign=1
    beta0=beta.hat
  }
beta_hat.1[bd] = beta0

beta0=0.01
  sign=0
  while(sign==0){
 L1=0
 L2=0

 for (i in 1:n){ if (cvlabel[i]==1) next
   if((num[i]>0)&&(delta[i]==1)&&(x[i]<=tau)){
   from=ord[i]+1
   to=ord[i]+num[i]
   include=from:to

temp00 = sum(Kh(h[bd],x[i]-observe)*id(x[i],observe)*yt(xx,x[i]) * exp(beta0*covariate))
temp11 = sum(Kh(h[bd],x[i]-observe)*id(x[i],observe)*yt(xx,x[i]) * covariate * exp(beta0*covariate))
temp22 = sum(Kh(h[bd],x[i]-observe)*id(x[i],observe)*yt(xx,x[i]) * covariate^2 * exp(beta0*covariate))

if(temp00 == 0) next
   L1=L1+sum(Kh(h[bd],x[i]-observe[include])*id(x[i],observe[include])*(covariate[include]-temp11/temp00))
   L2=L2+sum(Kh(h[bd],x[i]-observe[include])*id(x[i],observe[include])[]*(temp11^2-temp22*temp00)/(temp00^2))
   }
 }

if (is.na(L2)) return
beta.hat = beta0-L1/L2
    if (abs(beta0-beta.hat)<10^-3)
      sign=1
    beta0=beta.hat
  }
beta_hat.2[bd] = beta0;

hatv.1[bd] = n*h[bd]*(beta_hat.1[bd] - beta_hat.2[bd])^2/4;
print(bd)
}

slope.1 = lm(beta_hat.0~ h)$coef[2]

temp = slope.1^2*h^2 + hatv.1 
opt_h = which(temp ==min(temp[temp>0]))

 beta0=0.01
  sign=0
  while(sign==0){
 L1=0
 L2=0

 for (i in 1:n){
   if((num[i]>0)&&(delta[i]==1)&&(x[i]<=tau)){
   from=ord[i]+1
   to=ord[i]+num[i]
   include=from:to

temp00 = sum(Kh(h[opt_h],x[i]-observe)*id(x[i],observe)*yt(xx,x[i]) * exp(beta0*covariate))
temp11 = sum(Kh(h[opt_h],x[i]-observe)*id(x[i],observe)*yt(xx,x[i]) * covariate * exp(beta0*covariate))
temp22 = sum(Kh(h[opt_h],x[i]-observe)*id(x[i],observe)*yt(xx,x[i]) * covariate^2 * exp(beta0*covariate))

if(temp00 == 0) next
   L1=L1+sum(Kh(h[opt_h],x[i]-observe[include])*id(x[i],observe[include])*(covariate[include]-temp11/temp00))
   L2=L2+sum(Kh(h[opt_h],x[i]-observe[include])*id(x[i],observe[include])*(temp11^2-temp22*temp00)/(temp00^2))
   }
 }

if (is.na(L2)) return
beta.hat = beta0-L1/L2


 #   beta.hat=f(beta0)
    if (abs(beta0-beta.hat)<10^-3)
      sign=1
    beta0=beta.hat
  }

beta=c(beta,beta0)
#  se=c(se,fse(beta0))
 L11=0
 L2=0
 for (i in 1:n){
   if((num[i]>0)&&(delta[i]==1)&&(x[i]<=tau)){
   from=ord[i]+1
   to=ord[i]+num[i]
   include=from:to
temp0 = sum(Kh(h[opt_h],x[i]-observe)*id(x[i],observe)*yt(xx,x[i]) * exp(beta0*covariate))
temp1 = sum(Kh(h[opt_h],x[i]-observe)*id(x[i],observe)*yt(xx,x[i]) * covariate * exp(beta0*covariate))
temp2 = sum(Kh(h[opt_h],x[i]-observe)*id(x[i],observe)*yt(xx,x[i]) * covariate^2 * exp(beta0*covariate))


if (temp0==0) next
   L11=L11+(sum(Kh(h[opt_h],x[i]-observe[include])*id(x[i],observe[include])*(covariate[include]-temp1/temp0)))^2
   L2=L2+sum(Kh(h[opt_h],x[i]-observe[include])*id(x[i],observe[include])*(temp1^2-temp2*temp0)/(temp0^2))
   }
 }

se = c(se, sqrt(L11/(L2^2)))
#####################################################
num.rep = num.rep +1 
print(num.rep)
}

indicator=beta
indicator[abs(beta-beta.real)/se<=1.96]=1
indicator[abs(beta-beta.real)/se>1.96]=0
mean(indicator)*100
mean(se)
sqrt(var(beta))
mean(beta-beta.real)/beta.real
mean(beta-beta.real)
beta.real
mean(censoring)
n

